
#ifndef GEISS_TEXTURE_MANAGER
#define GEISS_TEXTURE_MANAGER 1

#define NUM_TEX 16

#include <d3d8.h>
#include "evallib/eval.h"
#include "md_defines.h"

#define TEXMGR_ERROR_MASK                 0x0F
#define TEXMGR_ERR_SUCCESS                0
#define TEXMGR_ERR_BAD_INDEX              1
/*
#define TEXMGR_ERR_OPENING                2
#define TEXMGR_ERR_IMAGE_NOT_24_BIT       3
#define TEXMGR_ERR_IMAGE_TOO_LARGE        4
#define TEXMGR_ERR_CREATESURFACE_FAILED   5
#define TEXMGR_ERR_LOCKSURFACE_FAILED     6
#define TEXMGR_ERR_CORRUPT_JPEG           7
*/
#define TEXMGR_ERR_FORMAT                 8
#define TEXMGR_ERR_BADFILE                9
#define TEXMGR_ERR_OUTOFMEM               10
#define TEXMGR_WARNING_MASK               0xF0
#define TEXMGR_WARN_ERROR_IN_INIT_CODE    0x10
#define TEXMGR_WARN_ERROR_IN_REG_CODE     0x20


typedef struct
{
	LPDIRECT3DTEXTURE8     pSurface;
	int                    img_w, img_h;
    /*
	int                    tex_w, tex_h;
	float                  scale_x, scale_y;  // the factors by which the original image was squished to become (img_w x img_h) texels in size.
	DDPIXELFORMAT          ddpf;
    */
	char				   szFileName[512];	
	float				   fStartTime;
	int                    nStartFrame;
	int                    nUserData;

	// stuff for expressions:
	char            m_szExpr[8192];         // for expression eval
    int				m_codehandle;	        // for expression eval
    varType         m_vars[EVAL_MAX_VARS];	// for expression eval
	// input variables for expression eval
    double          *var_time, *var_frame, *var_fps, *var_progress;
	double          *var_bass, *var_bass_att, *var_mid, *var_mid_att, *var_treb, *var_treb_att;
	// output variables for expression eval
	double          *var_x, *var_y;
	double          *var_sx, *var_sy, *var_rot, *var_flipx, *var_flipy;
	double          *var_r, *var_g, *var_b, *var_a;
	double          *var_blendmode;
	double          *var_repeatx, *var_repeaty;
	double          *var_done, *var_burn;
}
td_tex;

class texmgr
{
public:
	texmgr();
	~texmgr();

	// members
	void Init(LPDIRECT3DDEVICE8 lpDD);           // DirectDraw object
	int  LoadTex(char *szFilename, int iSlot, char *szInitCode, char *szCode, float time, int frame, unsigned int ck);
	void KillTex(int iSlot);
	void Finish();

	// data
	td_tex          m_tex[NUM_TEX];
	
protected:
	// members
	//bool TryCreateDDrawSurface(int iSlot, int w, int h);
	void FreeVars(int iSlot);
	void FreeCode(int iSlot);
	void RegisterBuiltInVariables(int iSlot);
	bool RunInitCode(int iSlot, char *szInitCode);
	bool RecompileExpressions(int iSlot);
	void StripLinefeedCharsAndComments(char *src, char *dest);

	// data
	LPDIRECT3DDEVICE8 m_lpDD;					
};




#endif